/*
	callbacks.c
*/

#include <gdk/gdk.h>
#include <glib.h>
#include <gtk/gtk.h>
#include "callbacks.h"
#include "config.h"
#include "graph.h"
#include "interface.h"
#include "main.h"


gboolean mainWin_delete (GtkWidget *win, GdkEvent *event, gpointer data)
{
	return FALSE;
}

void mainWin_destroy (GtkObject *object, gpointer data)
{
	gtk_main_quit ();
}

gboolean drawingarea_expose (GtkWidget *widget, GdkEventExpose *ev,
							gpointer data)
{
	graph_draw (mainGraph);
	return FALSE;
}

gboolean drawingarea_button_press (GtkWidget *widget,
				GdkEventButton *event, gpointer data)
{
	vertex_t *v;

	if (event->button == 3) {
		create_popupMenu ();
		return FALSE;
	} else if (event->button != 1)
		return FALSE;

	v = graph_find_vertex (mainGraph, event->x, event->y);

	if (!(event->state & GDK_SHIFT_MASK))
		graph_unselect_all (mainGraph);

	if (!v) {
		if (event->state & GDK_CONTROL_MASK) {
			v = (vertex_t *) g_malloc (sizeof (vertex_t));
			v->label = NULL;
			v->x = event->x;
			v->y = event->y;
			graph_add_vertex (mainGraph, v);
		}
		graph_draw (mainGraph);
		return FALSE;
	}

	if (event->state & GDK_CONTROL_MASK)
		graph_remove_vertex (mainGraph, v);
	else if (event->state & GDK_SHIFT_MASK)
		v->selected = !v->selected;
	else if (!v->selected)
		v->selected = TRUE;

	graph_draw (mainGraph);
	return FALSE;
}

gboolean drawingarea_motion_notify (GtkWidget *widget,
				GdkEventMotion *event, gpointer data)
{
	static gint last_x = -1, last_y = -1;

	if (!(event->state & GDK_BUTTON1_MASK)) {
		/* Button not down => nothing to do */
		last_x = -1;
		last_y = -1;
		return FALSE;
	}

	if (last_x == -1) {
		/* First motion event; record start */
		last_x = event->x;
		last_y = event->y;
		return FALSE;
	}

	/* Move stuff */
	graph_move_selected (mainGraph, event->x - last_x,
						event->y - last_y);
	last_x = event->x;
	last_y = event->y;

	graph_draw (mainGraph);
	return FALSE;
}

void about_activate (GtkMenuItem *menuitem, gpointer data)
{
	modal_message ("About...",
		"G r a p h T h i n g\n\n"
		"Version " GT_VERSION "\n\n"
		"(c) 2001 David Symonds"
			);
}

void add_edge_activate (GtkMenuItem *menuitem, gpointer data)
{
	GSList *vertices;
	edge_t *e;

	vertices = graph_find_selected (mainGraph);
	if (g_slist_length (vertices) != 2) {
		modal_message ("WTF?", "add_edge called badly!");
		return;
	}

	e = (edge_t *) g_malloc (sizeof (edge_t));
	e->from = (vertex_t *) vertices->data;
	e->to = (vertex_t *) vertices->next->data;
	e->directed = FALSE;
	e->weight = 1;
	graph_add_edge (mainGraph, e);

	g_slist_free (vertices);
	graph_draw (mainGraph);
}

void remove_edge_activate (GtkMenuItem *menuitem, gpointer data)
{
	GSList *vertices, *l;
	vertex_t *v1, *v2;

	vertices = graph_find_selected (mainGraph);
	if (g_slist_length (vertices) != 2) {
		modal_message ("WTF?", "remove_edge called badly!");
		return;
	}

	v1 = (vertex_t *) vertices->data;
	v2 = (vertex_t *) vertices->next->data;
	for (l = v1->edges; l; l = l->next) {
		edge_t *e = (edge_t *) l->data;
		if ((e->from == v2) || (e->to == v2)) {
			graph_remove_edge (mainGraph, e);
			break;
		}
	}

	g_slist_free (vertices);
	graph_draw (mainGraph);
}

void check_sanity (void *graph)
{
	int oops;
	char buf[64];

	status_push ("Checking sanity...", 5);

	oops = graph_sanity ((graph_t *) graph);
	if (oops == 0)
		modal_message ("Sanity", "No problems found.\n\n"
							"Graph is sane.");
	else {
		g_snprintf (buf, 64, "%i problems found!\n\n"
					"Graph is not sane!", oops);
		modal_message ("Sanity", buf);
	}

	status_pop (5);
}
