/*
	graph.h
*/

#ifndef __GRAPH_H__
#define __GRAPH_H__

#include <gdk/gdk.h>
#include <glib.h>
#include <gtk/gtk.h>

/*
 * DEFINES
 */

#define VERTEX_RADIUS	10

typedef struct {
	gchar *label;
	int x, y;
	gboolean selected;
	void *mark;
	GSList *edges;		/* edges incident with this vertex */
	int degree;		/* g_slist_length (edges) */
} vertex_t;

typedef struct {
	vertex_t *from, *to;
	gboolean directed;
	int weight;
	gboolean selected;
	void *mark;
} edge_t;

typedef struct {
	GSList *vertices;
	GSList *edges;
} graph_t;

typedef struct {
	int type;
	void *param1, *param2, *param3;
} search_t;

#define SEARCH_POSITION		0
#define SEARCH_SELECTED		1

/*
 * FUNCTIONS
 */

void graph_init (GtkWidget *widget);
void graph_close (void);
graph_t *graph_create (void);
void graph_destroy (graph_t *g);
void graph_load (graph_t *g);
void graph_load_file (GtkWidget *button, gpointer data);
void graph_save (graph_t *g);
void graph_save_file (GtkWidget *button, gpointer data);
void graph_clear (graph_t *g);
void graph_draw (graph_t *g);
void graph_add_vertex (graph_t *g, vertex_t *v);
void graph_add_edge (graph_t *g, edge_t *e);
void graph_add_vertex_dup (graph_t *g, vertex_t *v);
void graph_add_edge_dup (graph_t *g, edge_t *e);
void graph_remove_vertex (graph_t *g, vertex_t *v);
void graph_remove_edge (graph_t *g, edge_t *e);
vertex_t *graph_find_vertex (graph_t *g, int x, int y);
vertex_t *graph_find_vertex_by_name (graph_t *g, char *label);
GSList *graph_find_selected (graph_t *g);
gboolean graph_is_adjacent (vertex_t *v1, vertex_t *v2);
void graph_unselect_all (graph_t *g);
void graph_move_selected (graph_t *g, int dx, int dy);

void graph_v_remover (gpointer data, gpointer user);
void graph_e_remover (gpointer data, gpointer user);
void graph_v_drawer (gpointer data, gpointer user);
void graph_e_drawer (gpointer data, gpointer user);
gint graph_v_finder (gpointer data, gpointer user);

int graph_sanity (graph_t *g);

#endif	/* __GRAPH_H__ */
