/*
	matrix.h
*/

#ifndef __MATRIX_H__
#define __MATRIX_H__

/*
 * DEFINES
 */

typedef struct {
	int order;
	int *data;		/* stored in row-major order */
} matrix_t;

typedef struct {
	int terms;
	int *data;		/* x^1 term first */
} poly_t;

#define MAT(mat,row,col)	\
		((mat)->data[(row) * (mat)->order + col])

/*
 * FUNCTIONS
 */

matrix_t *matrix_new (int order);
void matrix_destroy (matrix_t *m);
void matrix_add (matrix_t *dst, matrix_t *a, matrix_t *b);
void matrix_sub (matrix_t *dst, matrix_t *a, matrix_t *b);
void matrix_mul (matrix_t *dst, matrix_t *a, matrix_t *b);

poly_t *poly_new (int terms);
void poly_destroy (poly_t *p);
void poly_add (poly_t *dst, poly_t *a, poly_t *b);
void poly_sub (poly_t *dst, poly_t *a, poly_t *b);
void poly_mul (poly_t *dst, poly_t *a, poly_t *b);
void poly_binomial (poly_t *dst, int power, int addend);
int poly_eval (poly_t *p, int x);

#endif	/* __MATRIX_H__ */
